<?php declare(strict_types=1);

namespace App\Entity;

use ApiPlatform\Metadata\ApiResource;
use App\Repository\OrderItemRepository;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Bridge\Doctrine\IdGenerator\UlidGenerator;
use Symfony\Bridge\Doctrine\Types\UlidType;
use Symfony\Component\Serializer\Annotation\Groups;
use Symfony\Component\Uid\Ulid;

#[ApiResource]
#[ORM\Entity(repositoryClass: OrderItemRepository::class)]
class OrderItem
{
    #[Groups(['food_order:latest'])]
    #[ORM\Column(length: 255, options: [
        'default' => 'nobody',
    ])]
    private string|null $createdBy = 'nobody';

    #[Groups(['food_order:latest'])]
    #[ORM\Column(length: 255, nullable: true)]
    private string|null $extras = null;

    #[ORM\JoinColumn(nullable: true)]
    #[ORM\ManyToOne(inversedBy: 'orderItems')]
    private FoodOrder|null $foodOrder = null;

    #[Groups('food_order:latest')]
    #[ORM\Column(type: 'boolean', options: [
        'default' => false,
    ])]
    private bool $isPaid = false;

    #[Groups(['food_order:latest'])]
    #[ORM\JoinColumn(nullable: false)]
    #[ORM\ManyToOne]
    private MenuItem|null $menuItem = null;

    #[Groups(['food_order:latest'])]
    #[ORM\Column(length: 255)]
    private string|null $name = null;

    #[Groups('food_order:latest')]
    #[ORM\Column(type: 'integer', options: [
        'default' => 0,
    ])]
    #[Positive]
    private int $priceCents = 0;

    public function __construct(
        #[Groups(['food_order:latest'])]
        #[ORM\Column(type: UlidType::NAME, unique: true)]
        #[ORM\CustomIdGenerator(class: UlidGenerator::class)]
        #[ORM\GeneratedValue(strategy: 'CUSTOM')]
        #[ORM\Id]
        private Ulid|null $id = new Ulid
    ) {
        $this->id ??= new Ulid;
    }

    public function getCreatedBy(): string|null
    {
        return $this->createdBy;
    }

    public function getExtras(): string|null
    {
        return $this->extras;
    }

    public function getFoodOrder(): FoodOrder|null
    {
        return $this->foodOrder;
    }

    public function getId(): Ulid|null
    {
        return $this->id;
    }

    public function getMenuItem(): MenuItem|null
    {
        return $this->menuItem;
    }

    public function getName(): string|null
    {
        return $this->name;
    }

    public function getPriceCents(): int
    {
        return $this->priceCents;
    }

    public function isPaid(): bool
    {
        return $this->isPaid;
    }

    public function setCreatedBy(string $createdBy): static
    {
        $this->createdBy = $createdBy;

        return $this;
    }

    public function setExtras(string|null $extras): static
    {
        $this->extras = $extras;

        return $this;
    }

    public function setFoodOrder(FoodOrder|null $foodOrder): static
    {
        $this->foodOrder = $foodOrder;

        return $this;
    }

    public function setIsPaid(bool $isPaid): self
    {
        $this->isPaid = $isPaid;
        return $this;
    }

    public function setMenuItem(MenuItem|null $menuItem): static
    {
        if (! $menuItem instanceof MenuItem) {
            return $this;
        }
        $this->menuItem = $menuItem;
        $this->name = $menuItem->getName();

        return $this;
    }

    public function setName(string $name): static
    {
        $this->name = $name;

        return $this;
    }

    public function setPriceCents(int $priceCents): self
    {
        $this->priceCents = $priceCents;
        return $this;
    }
}
